/**
 * Copyright (c) 2011, REMPL.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1) Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 * 2) Redistributions in binary form must reproduce the above
 * copyright notice, this list of conditions and the following
 * disclaimer in the documentation and/or other materials provided
 * with the distribution.
 * 3) Neither the name of the REMPL.com nor the names of its
 * contributors may be used to endorse or promote products derived
 * from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package com.rempl.reporters.pdd;

// REMPL API from com.rempl:rempl-api classes
import com.rempl.api.Element;
import com.rempl.api.Logger;
import com.rempl.api.Model;
import com.rempl.api.Rempler;
import com.rempl.api.Reporter;
import com.rempl.api.RuntimeProblem;

// Bridge API from com.rempl.readers.its:bridge-its-api
import com.rempl.readers.its.bridge.BridgeFactory;
import com.rempl.readers.its.bridge.CodeUrlBuilder;
import com.rempl.readers.its.bridge.IssueUrlBuilder;

// file management
import java.io.File;

// collection management
import java.util.Collection;
import java.util.Map;

// files manipulation
import org.apache.commons.io.FileUtils;

/**
 * Reporter of PDD puzzles.
 *
 * @author Yegor Bugayenko (yegor@rempl.com)
 * @version $Id: PDDReporter.java 1389 2011-05-23 23:07:59Z guard $
 */
public final class PDDReporter implements Reporter {

    /**
     * The {@link Rempler} we're working for.
     * @see #setRempler(Rempler)
     */
    private Rempler rempler;

    /**
     * Public ctor.
     */
    public PDDReporter() {
        this.rempler = null;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void setRempler(final Rempler rpr) {
        this.rempler = rpr;
        Logger.debug(
            this,
            "#setRempler(%s)",
            rpr.getClass().getName()
        );
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void report(final Model model, final File dir)
        throws java.io.IOException {
        final Wbs wbs = this.wbs();
        final Collection<Element> issues =
            model.query("context Issue inv: true");
        for (Element issue : issues) {
            wbs.add(new UmlIssue(issue, model));
        }
        final Collection<Element> puzzles =
            model.query("context Puzzle inv: true");
        for (Element puzzle : puzzles) {
            wbs.add(new UmlPuzzle(puzzle, model));
        }
        final File xml = new File(dir, "pdd.xml");
        this.xml(wbs, xml);
        Logger.info(
            this,
            "#report(%s, '%s'): %d issues, %d puzzles, "
            + "%d bytes as XML written to %s",
            model.getClass().getName(),
            dir.getPath(),
            issues.size(),
            puzzles.size(),
            xml.length(),
            xml.getPath()
        );
        final File index = new File(dir, "index.html");
        new Transformer().transform(xml, index);
        Logger.info(
            this,
            "#report(%s, '%s'): XML converted to %d bytes in HTML at %s",
            model.getClass().getName(),
            dir.getPath(),
            index.length(),
            index.getPath()
        );
    }

    /**
     * Create and configure the WBS.
     * @return The WBS to work with
     */
    private Wbs wbs() {
        final Wbs wbs = new Wbs();
        final Map<String, String> params =
            (Map<String, String>) this.rempler.configuration()
                .get("/*/its/params/*", Map.class);
        final IssueUrlBuilder issueUrlBuilder =
            BridgeFactory.INSTANCE.makeIssueUrlBuilder(
                this.rempler.configuration()
                    .get("/*/its/issueUrlBuilder/text()")
            );
        final CodeUrlBuilder codeUrlBuilder =
            BridgeFactory.INSTANCE.makeCodeUrlBuilder(
                this.rempler.configuration()
                    .get("/*/its/codeUrlBuilder/text()")
            );
        try {
            issueUrlBuilder.configure(params);
            codeUrlBuilder.configure(params);
        } catch (com.rempl.readers.its.bridge.BridgeException ex) {
            throw RuntimeProblem.make(ex);
        }
        wbs.setIssueUrlBuilder(issueUrlBuilder);
        wbs.setCodeUrlBuilder(codeUrlBuilder);
        Logger.debug(
            this,
            "#wbs(): created WBS %s",
            wbs.getClass().getName()
        );
        return wbs;
    }

    /**
     * Create XML out of the {@link Wbs}.
     * @param wbs The WBS to work with
     * @param xml The XML document to be created
     * @see #report(Model, File)
     * @throws java.io.IOException If some I/O problem occurs
     */
    private void xml(final Wbs wbs, final File xml)
        throws java.io.IOException {
        if (!xml.exists() && xml.createNewFile()) {
            Logger.info(
                this,
                "#xml(%s, %s): new XML file created",
                wbs.getClass().getName(),
                xml.getPath()
            );
        }
        FileUtils.writeStringToFile(xml, wbs.toXml());
    }

}
