/**
 * Copyright (c) 2011, REMPL.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1) Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 * 2) Redistributions in binary form must reproduce the above
 * copyright notice, this list of conditions and the following
 * disclaimer in the documentation and/or other materials provided
 * with the distribution.
 * 3) Neither the name of the REMPL.com nor the names of its
 * contributors may be used to endorse or promote products derived
 * from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package com.rempl.reporters.pdd;

// REMPL API from com.rempl:rempl-api classes
import com.rempl.api.Element;
import com.rempl.api.Model;

// localization
import java.util.Collection;
import java.util.Locale;

// for MD5 hashing
import org.apache.commons.codec.digest.DigestUtils;

/**
 * One puzzle.
 *
 * @author Yegor Bugayenko (yegor@rempl.com)
 * @version $Id: UmlPuzzle.java 1574 2011-07-06 23:16:29Z guard $
 * @see PDDReporter#report(Model, File)
 */
final class UmlPuzzle implements Puzzle {

    /**
     * Length of MD5 digest.
     */
    private static final Integer DIGEST_LENGTH = 8;

    /**
     * The subject.
     * @see UmlPuzzle(Element)
     */
    private final String subject;

    /**
     * The subject.
     * @see UmlPuzzle(Element)
     */
    private final String body;

    /**
     * The subject.
     * @see UmlPuzzle(Element)
     */
    private final String file;

    /**
     * The subject.
     * @see UmlPuzzle(Element)
     */
    private final String revision;

    /**
     * The subject.
     * @see UmlPuzzle(Element)
     */
    private final String lines;

    /**
     * The subject.
     * @see UmlPuzzle(Element)
     */
    private final Integer estimate;

    /**
     * The subject.
     * @see UmlPuzzle(Element)
     */
    private final String issueName;

    /**
     * Public ctor.
     * @param element The element from Model
     * @param model The model to work in
     * @see PDDReporter#report(Model, File)
     * @todo #18 This is just a stub for now. We need to get subject
     *           information from the puzzledElement, and understand
     *           it properly. That could be a file, or a class, or
     *           a method, or something else. Revision has to be
     *           retrieved from reader-scm UML information (which is
     *           not implemented yet).
     */
    public UmlPuzzle(final Element element, final Model model) {
        this.subject = this.file(element, model);
        this.file = this.subject;
        this.revision = this.revision(element, model);

        this.body = element.get("body");
        this.issueName = element.get("ticket");
        this.estimate = Integer.valueOf(element.get("estimate"));
        this.lines = element.get("lines");
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public String getSubject() {
        return this.subject;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public String getBody() {
        return this.body;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public String getFile() {
        return this.file;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public String getRevision() {
        return this.revision;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public String getLines() {
        return this.lines;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public String getHash() {
        final String baseline =
            this.issueName
            + ":"
            + this.body.replaceAll("[^a-zA-Z]", "")
                .toUpperCase(Locale.ENGLISH);
        return DigestUtils.md5Hex(baseline)
            .substring(0, this.DIGEST_LENGTH)
            .toUpperCase(Locale.ENGLISH);
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public Integer getEstimate() {
        return this.estimate;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public String getIssueName() {
        return this.issueName;
    }

    /**
     * Get file name of the puzzle.
     * @param puzzle The Puzzle UML element
     * @param model The model to work with
     * @return The name of the file
     * @see UmlPuzzle(Element,Model)
     * @todo #18 This implementation is very preliminary
     */
    private String file(final Element puzzle, final Model model) {
        final Collection<Element> artifacts =
            model.query("context Artifact inv: true");
        for (Element artifact : artifacts) {
            if (puzzle.refsTo("puzzledElement", artifact)) {
                return artifact.get("fileName");
            }
        }
        return "unknown";
    }

    /**
     * Get revision name of the puzzle.
     * @param puzzle The Puzzle UML element
     * @param model The model to work with
     * @return The revision of the file
     * @see UmlPuzzle(Element,Model)
     * @todo #18 This implementation is very preliminary
     */
    private String revision(final Element puzzle, final Model model) {
        return null;
    }

}
